import { Modal, Form, Select, Input, Radio } from 'antd'
import { forwardRef, useImperativeHandle, useState } from 'react'
import { adTypes } from '../../const'
import styles from './index.module.css'
import type { RowDataType } from '../ContentTable/types'
import type { 
  PropsType, Action, FormDataType, 
  OptionsItemType, RadioOptionsType 
} from './types'

const { Item, useForm } = Form
const adTypesOptions:RadioOptionsType = Object.entries(adTypes).map(([value,label])=>({value: +value,label}))
const initialValues = {
  unionAdPosition: 0, // 广告类型
}

const CreateModal = forwardRef(({ allAppList, onCreate, onUpdate }:PropsType, ref) => {
  const [isOpen, setIsOpen] = useState(false)
  const [action, setAction] = useState<Action>()
  const [form] = useForm()
  const [editRow, setEditRow] = useState<RowDataType>()
  const isEdit = action === 'edit'
  const title = isEdit ? '编辑广告' : '新建广告'
  const allAppOptions:OptionsItemType[] = allAppList.map(({id, unionAppName})=>({label:unionAppName, value:id}))

  useImperativeHandle(ref,()=>({
    open: (editRow?:RowDataType) => {
      setAction(editRow?'edit':'create')
      setEditRow(editRow)
      if (editRow) {
        const {
          applicationId, // 应用id
          unionAdName, // 广告名称
          unionAdPosition // 广告类型
        } =  editRow 
        form.setFieldsValue({
          applicationId,
          unionAdName,
          unionAdPosition 
        })
      }
      setIsOpen(true)
    }
  }))

  const onOk = async () => {
    try {
      const res:FormDataType =  await form.validateFields()
      switch (action) {
        case 'create':
          onCreate(res, ()=>{setIsOpen(false)})
          break
        case 'edit':
          onUpdate({...(editRow as RowDataType), ...res},()=>{setIsOpen(false)})
      }
    } catch (error) { /* empty */ }
  }

  const onCancel = () => {
    setIsOpen(false)
  }

  const afterOpenChange = (isOpen: boolean) => {
    if (!isOpen) {
      form.resetFields()
    }
  }

  return (
    <Modal
      className={styles['custom-modal']}
      centered={true}
      open={isOpen}
      title={title}
      width={500}
      onCancel={onCancel}
      onOk={onOk}
      afterOpenChange={afterOpenChange}
    >
      <Form
        form={form}
        initialValues={initialValues}
        labelCol={{ span: 5 }}
        wrapperCol={{ span: 18, offset: 1}}
        colon={ false }
        requiredMark={ false }
      >
        <Item
          name='applicationId'
          label='应用'
          rules={[{ required: true, message: '请选择应用' }]}
        >
          <Select 
            placeholder='请选择应用'
            options={allAppOptions}
          />
        </Item>
        <Item
          name='unionAdName'
          label='广告名称'
          rules={[{ required: true, message: '请输入广告名称' }]}
        >
          <Input placeholder='请输入广告名称' showCount maxLength={ 20 } />
        </Item>
        <Item
          name='unionAdPosition'
          label='广告类型'
          rules={[{ required: true, message: '请选择广告类型' }]}
        >
          <Radio.Group options={adTypesOptions} />
        </Item>
      </Form>
    </Modal>
  )
})

export default CreateModal
